#!/bin/bash
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# Copyright (c) 2019-2024, AllWorldIT
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


autorun_cmdline() {
	local param
	for param in $(< /proc/cmdline); do
		case "${param}" in
			autorun=*) echo "${param#*=}" ; return 0 ;;
		esac
	done
}

autorun() {
	local autorun script

	# Last we check for a script in /root
	if [ -f /root/autorun ]; then
		autorun=/root/autorun
	fi

	# Next we check the /host folder
	if [ -f /host/autorun ]; then
		autorun=/host/autorun
	fi

	# Next we check the /OEM folder
	if [ -f /OEM/autorun ]; then
		autorun=/OEM/autorun
	fi

	script="$(autorun_cmdline)"
	if [ -n "$script" ]; then
		if [[ "$script" =~ ^https?:// ]]; then
			while true; do
				echo "Downloading autorun script '$script'..."
				if wget "${script}" --retry-connrefused -O /root/autorun; then
					break
				fi
				echo "ERROR: Download failed... retrying in 5s"
				sleep 5
			done
			autorun=/root/autorun
		elif [[ "$script" =~ ^/ ]]; then
			autorun="$script"
		else
			echo "ERROR: Kernel commandline argument to autorun '$script' does not begin with '/', 'http://' or 'https://'"
			read -r
			exit 1
		fi
	fi

	# If script doesn't exist, return
	if [ ! -f "$autorun" ]; then
		return
	fi

	# Try set script executable
	if [ ! -x "$autorun" ]; then
		chmod +x "$autorun" || :
	fi

	if [ ! -x "$autorun" ]; then
		echo "ERROR: Autorun script '$autorun' is not executable"
		read -r
		exit 1
	fi

	exec "$autorun"
}

# We only run on tty1 if we're not running serial
if [ "$(tty)" = "/dev/tty1" ] && ! grep -q console=ttyS0 /proc/cmdline; then
	autorun
fi

# And we only run on serial if we have console=ttyS0 in /proc/cmdline
if [ "$(tty)" = "/dev/ttyS0" ] && grep -q console=ttyS0 /proc/cmdline; then
	autorun
fi
